#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <net/ethernet.h>
#include <netinet/in.h>
#include <netinet/ip.h>

#include "pktlib.h"
#include "pktbuf.h"
#include "pcap.h"

#define SPECIAL_CRYPT_PROTO_FLAG 0x80

static pktbuf_t decrypt_ip(pktbuf_t pktbuf, int key)
{
  char *p;
  struct ip *iphdr;
  int hdrsize, paysize;
  int i;

  p = pktbuf_get_header(pktbuf);

  if (pktbuf_get_size(pktbuf) < sizeof(*iphdr))
    return pktbuf;

  iphdr = (struct ip *)p;
  hdrsize = iphdr->ip_hl << 2;
  paysize = ntohs(iphdr->ip_len) - hdrsize;

  if (!(iphdr->ip_p & SPECIAL_CRYPT_PROTO_FLAG))
    return pktbuf;

  if (pktbuf_get_size(pktbuf) < hdrsize + paysize)
    return pktbuf;

  for (i = 0; i < paysize; i++) {
    (p + hdrsize)[i] ^= key;
  }

  iphdr->ip_p &= ~SPECIAL_CRYPT_PROTO_FLAG;

  return pktbuf;
}

int main(int argc, char *argv[])
{
  pktbuf_t pktbuf;
  int key, size, linktype, origsize;
  struct timeval tm;
  struct ether_header *ehdr;

  argc--; argv++;

  key = atoi(argv[0]);

  pkt_pcap_read(stdin, NULL, 0, &linktype, NULL, NULL); /* 󥯥פμ */

  while (1) {
    pktbuf = pktbuf_create(pktlib_linktype_offset(linktype));
    size = pkt_pcap_read(stdin, pktbuf_get_header(pktbuf), 80 * 1024,
			 &linktype, &origsize, &tm); /* pcap */
    if (size < 0)
      break;
    pktbuf_set_size(pktbuf, size);

    ehdr = (struct ether_header *)pktbuf_get_header(pktbuf);

    if (ntohs(ehdr->ether_type) == ETHERTYPE_IP) {
      pktbuf_delete_header(pktbuf, ETHER_HDR_LEN);
      pktbuf = decrypt_ip(pktbuf, key);
      pktbuf = pktbuf_checksum_correct_ip(pktbuf);
      pktbuf_add_header(pktbuf, ETHER_HDR_LEN);
    }

    pkt_pcap_write(stdout, pktbuf_get_header(pktbuf), pktbuf_get_size(pktbuf),
                   linktype, pktbuf_get_size(pktbuf), &tm); /* pcap */
    fflush(stdout);

    pktbuf_destroy(pktbuf); /* ѥåȤѴ */
  }

  return 0;
}
